﻿# 用户身份认证系统（Identity）

 对于开发自有用户系统的身份认证需求而言，`AspNetCore Identity`已经是个很完美的身份认证框了。基本上身份认证系统中的各种需求在这个框架上都能得到很好的实现。下面列举一些常用的功能点：
 - 用户注册，登录，修改密码，重置密码
 - 邮箱/手机 验证码系统，二元验证
 - 第三方账号OAuth2登录
 - 基于Token的身份认证

OSharp的身份认证，基于`AspNetCore Identity`做了如下的工作：

- 使用AspNetCore原生的用户身份认证框架，身份认证相关操作统一使用UserManager<TUser>，RoleMamanger<TRole>两个入口，保持了原生Identity的体系强大性与功能完整性。
- 重新设计了用户存储UserStore和角色存储RoleStore，使用框架内设计的IRepository<TEntity,TKey>数据仓储接口来实现对数据的仓储操作，使Identity身份认证系统与框架完美结合，避免了使用官方的Microsoft.AspNetCore.Identity.EntityFrameworkCore造成多个上下文或者被强制使用Identity上下文作为系统数据上下文来实现业务造成的尴尬。

## 身份认证系统组成

- **User**：用户信息，一个`User`表示一个具有基本信息的自然人，只包含用户名、密码、昵称、邮箱、头像、手机号及一些身份认证所必需的信息。与业务相关的用户信息，都应通过关系表进行存储。一些额外的用户信息可存储中用户详细信息`UserDetail`中。
- **Role**：角色信息，表示用户在系统中的身份，一个用户具有了某种身份，才拥有相应的权限。用户与角色通过`UserRole`进行关联。
- **UserClaim**：用户声明信息，可用于存储一些额外的动态的用户信息
- **RoleClaim**：角色声明信息，可用于存储一些额外的动态的角色信息
- **UserLogin**：用户第三方登录信息，比如QQ登录，其中存储着第三方登录提供程序、第三方用户唯一标识、第三方用户昵称、关联的当前系统用户编号。
- **UserToken**：用户令牌信息，用于基于Token的身份认证

## 系统账号使用流程分析

账号来源分两种，一是通过新用户注册的方式创建的账号，二是通过第三方（如QQ）OAuth2登录方式创建的账号，两种途径在数据处理流程上略有不同。

### 新用户注册

- **Email**：为了确保注册用户可以使用**忘记密码**这些功能，一个唯一且可用的Email是必须的。
- **登录密码**：注册的用户，登录密码当然是需要的

因此，OSharp的新用户注册的默认流程规定如下：
- 使用`Email + 登录密码`进行注册，注册成功之后，使用Email作为登录账号使用
- 注册成功之后自动发送“Email激活邮箱”
- 用户接收邮件激活账号之后，才能正常登录
- 用户登录之后可以修改Email绑定，也要求唯一且可用
- 用户昵称可以随机生成，头像使用默认，这些信息可以登录之后进行完善

### 第三方OAuth2登录

第三方登录是基于用户在第三方平台上已有的账号和密码来快速完成己方应用的登录或者注册的功能。而这里的第三方平台，一般是已经拥有大量用户的平台，国外的比如Facebook，Twitter等，国内的比如微博、微信、QQ等。 第三方登录的目的是使用用户在其他平台上频繁使用的账号，来快速登录己方产品，也可以实现不注册就能登录。
通常第三方OAuth2平台会给我们返回一些基本信息：用户唯一标识字符串、显示的用户昵称、头像地址。用这些信息创建一个新用户，可以把唯一标识作为用户名，密码可以留空，不需要管理密码，Email也不是必须的。
因此，OSharp的第三方OAuth2登录的默认流程规定如下：
- 用户在登录界面选择第三方账号登录方式
- 跳转到第三方登录界面进行登录，返回用户基本信息
- 拿到用户基本信息后，返回前端让用户进行选择
  - 绑定现有账号，输入用户名密码进行登录，关联第三方登录信息
  - 直接创建新账号进行一键登录，使用唯一标识作为用户名创建新账号并关联第三方登录信息
