// -----------------------------------------------------------------------
//  <copyright file="BitmapToImageSourceConverter.cs" company="柳柳软件">
//      Copyright (c) 2017 66SOFT. All rights reserved.
//  </copyright>
//  <site>http://www.66soft.net</site>
//  <last-editor>郭明锋</last-editor>
//  <last-date>2017-08-08 13:20</last-date>
// -----------------------------------------------------------------------

namespace OSharp.Wpf.Converters;

public class BitmapToImageSourceConverter : IValueConverter
{
    /// <summary>
    /// 转换值。
    /// </summary>
    /// <returns>
    /// 转换后的值。如果该方法返回 null，则使用有效的 null 值。
    /// </returns>
    /// <param name="value">绑定源生成的值。</param>
    /// <param name="targetType">绑定目标属性的类型。</param>
    /// <param name="parameter">要使用的转换器参数。</param>
    /// <param name="culture">要用在转换器中的区域性。</param>
    public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
    {
        Bitmap bmp = value as Bitmap;
        if (bmp == null)
        {
            return null;
        }
        byte[] bytes = bmp.ToBytes();
        using (MemoryStream ms = new MemoryStream(bytes, false))
        {
            BitmapImage source = new BitmapImage();
            source.BeginInit();
            source.StreamSource = ms;
            source.CacheOption = BitmapCacheOption.OnLoad;
            source.EndInit();
            return source;
        }
    }

    /// <summary>
    /// 转换值。
    /// </summary>
    /// <returns>
    /// 转换后的值。如果该方法返回 null，则使用有效的 null 值。
    /// </returns>
    /// <param name="value">绑定目标生成的值。</param>
    /// <param name="targetType">要转换到的类型。</param>
    /// <param name="parameter">要使用的转换器参数。</param>
    /// <param name="culture">要用在转换器中的区域性。</param>
    public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
    {
        throw new NotImplementedException();
    }
}
